@@ -24,7 +24,7 @@ use std::{
 };
 
 /// The Size of each Buffer in the "BufferList"
-const BUFFER_SIZE: usize = 256;
+const BUFFER_SIZE: usize = 1024;
 
 mod node;
 use node::NodeState;
@@ -172,6 +172,8 @@ impl<T> Sender<T> {
         // Calculate the Starting-Location of the currently loaded
         // Buffer
         let mut start = (tmp_buffer.position_in_queue - 1) * BUFFER_SIZE;
+
+        let mut last_buffer = true;
         // If the Target-Location is before the current Buffer's start,
         // we need to move back in the List of Buffers until we find the one
         // that actually contains our Target-Location
@@ -180,6 +182,8 @@ impl<T> Sender<T> {
             tmp_buffer_ptr = tmp_buffer.previous as *mut BufferList<T>;
             tmp_buffer = ManuallyDrop::new(unsafe { Box::from_raw(tmp_buffer_ptr) });
 
+            last_buffer = false;
+
             // Recalculate the Buffers Starting position for the new one
             start = (tmp_buffer.position_in_queue - 1) * BUFFER_SIZE;
         }
@@ -189,10 +193,12 @@ impl<T> Sender<T> {
 
         // Actually store the Data into the Buffer at the previously
         // calculated Index
-        tmp_buffer.buffer[index].store(data);
+        unsafe { tmp_buffer.buffer.get_unchecked(index) }.store(data);
+
+        if last_buffer && index == 2 {
+            tmp_buffer.allocate_next(tmp_buffer_ptr, &self.tail_of_queue);
+        }
 
-        // TODO
-        // Possible optimization regarding to pre-allocate the next buffer early
         Ok(())
     }
 }
@@ -487,6 +493,37 @@ impl<T> Receiver<T> {
             };
         }
     }
+
+    /// Returns a RefIter for the Queue, this allows you to still use the
+    /// Queue-Receiver once the Iterator has been dropped
+    pub fn iter_mut<'queue, 'iter>(&'queue mut self) -> RefIter<'iter, T>
+    where
+        'queue: 'iter,
+    {
+        self.into_iter()
+    }
+}
+
+mod owned_iter;
+pub use owned_iter::OwnedIter;
+impl<T> IntoIterator for Receiver<T> {
+    type Item = T;
+    type IntoIter = OwnedIter<T>;
+
+    fn into_iter(self) -> Self::IntoIter {
+        OwnedIter::new(self)
+    }
+}
+
+mod ref_iter;
+pub use ref_iter::RefIter;
+impl<'queue, T> IntoIterator for &'queue mut Receiver<T> {
+    type Item = T;
+    type IntoIter = RefIter<'queue, T>;
+
+    fn into_iter(self) -> Self::IntoIter {
+        RefIter::new(self)
+    }
 }
 
 // These are both save to manually implement because we would garantuee that
@@ -612,4 +649,18 @@ mod tests {
         assert_eq!(Ok(13), rx.try_dequeue());
         assert_eq!(Err(DequeueError::Closed), rx.try_dequeue());
     }
+
+    #[test]
+    fn iter_mut() {
+        let (mut rx, tx) = queue::<usize>();
+
+        tx.enqueue(13).unwrap();
+        drop(tx);
+
+        let mut iter = (&mut rx).into_iter();
+        assert_eq!(Some(13), iter.next());
+        assert_eq!(None, iter.next());
+
+        assert_eq!(true, rx.is_closed());
+    }
 }
